/*
 * Copyright (C) 2003 Ranjan Ghosh <ghosh@linux>
 */

#include "regimp.h"

#include <qlabel.h>
#include <qrect.h>
#include <qtimer.h>

#include <kmainwindow.h>
#include <klocale.h>
#include <kcmdlineargs.h>

#include <X11/Xutil.h>
#include <X11/Intrinsic.h>

#include <iostream>
using namespace std;

#include <kwin.h>

void enumWindowsRecursively(Display* display, int screen, Window start, const char* name, Window enumarray[],int& windowcount) {
  Status stat;
  int n;
  unsigned int num;
  Window root;
  Window parent;
  Window* children;
  char *title;

  if (XFetchName(display,start,&title)==1) {
    if (strstr(title,name)) {
   enumarray[windowcount]=start;
   windowcount++;
   }
    XFree(title);
    }

  stat=XQueryTree(display,start,&root,&parent,&children,&num);

  if (stat==1) {
    //search each child window for a match:
    if (XFetchName(display,start,&title)==1) {
      if (strstr(title,name)) {
        enumarray[windowcount]=start;
        windowcount++;
      }
      XFree(title);
    }

    // search the descendents of each child for a match:
    for (n=num-1;n>=0;n--) enumWindowsRecursively(display,screen,children[n],name,enumarray,windowcount);
    if (children!=NULL) XFree(children);
  }
};

int enumWindows(Display* display,const char* name,Window enumarray[]) {
  int windowcount=0;
 	enumWindowsRecursively(display,XDefaultScreen(display),XDefaultRootWindow(display),name,enumarray,windowcount);
  return windowcount;
}

regimp::regimp(KCmdLineArgs* args)
    : KMainWindow( 0, "regimp", WType_TopLevel | WDestructiveClose )
{
    // set the shell's ui resource file
    setXMLFile("regimpui.rc");

    showMaximized();

    label1=new QLabel("",this,"label1");

   for (int j=0;j<100;j++) {
     windowx[j]=-1;
     windowy[j]=-1;
   }

 minimized=false;

	setFocusPolicy(QWidget::ClickFocus);

	Display* display;
 display=XOpenDisplay(NULL);
	windowcount=enumWindows(display,"GIMP",windowlist);

 if (args->count()==0) {
//   if (enum_window_count==0) system("gimp-1.3"); else activateGIMP();
   system("gimp-1.3 &");
   }

	XCloseDisplay(display);

 gimprunning=false;

 QTimer *timer=new QTimer(this);
 connect(timer,SIGNAL(timeout()),this,SLOT(timerDone()));
 timer->start(500,false);
}

regimp::~regimp()
{
}

void regimp::timerDone() {
 int curwcount;
	Display* display;
 display=XOpenDisplay(NULL);
	curwcount=enumWindows(display,"GIMP",secondlist);

 for (int i=0;i<curwcount;i++) {
   KWin::setState(secondlist[i],NET::SkipTaskbar);
 }

 XCloseDisplay(display);
 if ((curwcount==0) and (gimprunning)) close();
 if ((curwcount!=0) and (!gimprunning)) gimprunning=true;
}

void regimp::saveWindowPositions() {
	Display* display;
 display=XOpenDisplay(NULL);

	Window w,child,dummy;
 int wx,wy;
 bool success;

	windowcount=enumWindows(display,"GIMP",windowlist);

 for (int i=0;i<windowcount;i++) {
	  w=windowlist[i];
   child=w;
   wx=0;wy=0;
   success=XTranslateCoordinates(display,w,XDefaultRootWindow(display),0,0,&wx,&wy,&child);
   if (success) {
     if ((child!=None) && (child!=w))
      {
        XTranslateCoordinates (display,child,XDefaultRootWindow(display),0,0,&wx,&wy,&dummy);
      }
     windowx[i]=wx;
     windowy[i]=wy;
   } else {
     windowx[i]=-1;
     windowy[i]=-1;
   }

	}

 XCloseDisplay(display);
}

void regimp::hideEvent(QHideEvent *e) {
 saveWindowPositions();

	Display* display;
 Window w;
 display=XOpenDisplay(NULL);

 for (int i=0;i<windowcount;i++) {
	  w=windowlist[i];
   XUnmapWindow(display,w);
 }

	XCloseDisplay(display);
 minimized=true;

 KMainWindow::hideEvent(e);
}

void regimp::activateGIMP(bool repos) {
 Display* display;
 display=XOpenDisplay(NULL);

	Window w;

	for (int i=0;i<windowcount;i++) {
	  w=windowlist[i];

   KWin::setState(w,NET::SkipTaskbar);
   KWin::deIconifyWindow(w,false);
   KWin::setActiveWindow(w);
   XFlush(display);
   if ((windowx[i]!=-1) and (repos)) XMoveWindow(display,w,windowx[i],windowy[i]);
	}
	XCloseDisplay(display);
}

void regimp::focusInEvent(QFocusEvent *e) {
  KMainWindow::focusInEvent(e);
  if (!minimized) saveWindowPositions();
  activateGIMP(minimized);
  minimized=false;
}

void regimp::showEvent(QShowEvent *e) {
  KMainWindow::showEvent(e);

}

#include "regimp.moc"
